#![feature(prelude_import)]
//! KAS widget library
use kas::prelude::*;
use std::fmt::Debug;
/// A scrollable region
#[derive(Debug)]
pub struct ScrollRegion<W: Widget> {
    core: ::kas::CoreData,
    inner: W,
}
impl<W: Widget> Scrollable for ScrollRegion<W> {
    fn scroll_axes(&self, _: Size) -> (bool, bool) {
        Default::default()
    }
    #[inline]
    fn max_scroll_offset(&self) -> Offset {
        Default::default()
    }
    #[inline]
    fn scroll_offset(&self) -> Offset {
        Default::default()
    }
    #[inline]
    fn set_scroll_offset(&mut self, _: &mut EventMgr, _: Offset) -> Offset {
        Default::default()
    }
}
impl<W: Widget> ::kas::WidgetCore for ScrollRegion<W> {
    #[inline]
    fn id_ref(&self) -> &::kas::WidgetId {
        &self.core.id
    }
    #[inline]
    fn rect(&self) -> ::kas::geom::Rect {
        self.core.rect
    }
    #[inline]
    fn widget_name(&self) -> &'static str {
        "ScrollRegion"
    }
    #[inline]
    fn as_widget(&self) -> &dyn ::kas::Widget {
        self
    }
    #[inline]
    fn as_widget_mut(&mut self) -> &mut dyn ::kas::Widget {
        self
    }
}
impl<W: Widget> ::kas::WidgetChildren for ScrollRegion<W> {
    fn num_children(&self) -> usize {
        1usize
    }
    fn get_child(&self, _index: usize) -> Option<&dyn ::kas::Widget> {
        match _index {
            0usize => Some(&self.inner),
            _ => None,
        }
    }
    fn get_child_mut(&mut self, _index: usize) -> Option<&mut dyn ::kas::Widget> {
        match _index {
            0usize => Some(&mut self.inner),
            _ => None,
        }
    }
}
impl<W: Widget> ::kas::layout::AutoLayout for ScrollRegion<W> {
    fn size_rules(
        &mut self,
        size_mgr: ::kas::theme::SizeMgr,
        axis: ::kas::layout::AxisInfo,
    ) -> ::kas::layout::SizeRules {
        use ::kas::layout;
        (layout::Visitor::single(&mut self.inner)).size_rules(size_mgr, axis)
    }
    fn set_rect(&mut self, mgr: &mut ::kas::event::ConfigMgr, rect: ::kas::geom::Rect) {
        use ::kas::layout;
        self.core.rect = rect;
        (layout::Visitor::single(&mut self.inner)).set_rect(mgr, rect);
    }
    fn find_id(&mut self, coord: ::kas::geom::Coord) -> Option<::kas::WidgetId> {
        use ::kas::layout;
        if !self.rect().contains(coord) {
            return None;
        }
        let coord = coord + self.translation();
        (layout::Visitor::single(&mut self.inner))
            .find_id(coord)
            .or_else(|| Some(self.id()))
    }
    fn draw(&mut self, draw: ::kas::theme::DrawMgr) {
        use ::kas::layout;
        (layout::Visitor::single(&mut self.inner)).draw(draw);
    }
}
impl<W: Widget> ::kas::Layout for ScrollRegion<W> {
    fn size_rules(
        &mut self,
        size_mgr: ::kas::theme::SizeMgr,
        axis: ::kas::layout::AxisInfo,
    ) -> ::kas::layout::SizeRules {
        <Self as ::kas::layout::AutoLayout>::size_rules(self, size_mgr, axis)
    }
    fn set_rect(&mut self, mgr: &mut ::kas::event::ConfigMgr, rect: ::kas::geom::Rect) {
        <Self as ::kas::layout::AutoLayout>::set_rect(self, mgr, rect);
    }
    fn find_id(&mut self, coord: ::kas::geom::Coord) -> Option<::kas::WidgetId> {
        <Self as ::kas::layout::AutoLayout>::find_id(self, coord)
    }
    fn draw(&mut self, draw: ::kas::theme::DrawMgr) {
        <Self as ::kas::layout::AutoLayout>::draw(self, draw);
    }
}
impl<W: Widget> ::kas::Widget for ScrollRegion<W> {
    fn pre_configure(&mut self, _: &mut ::kas::event::ConfigMgr, id: ::kas::WidgetId) {
        self.core.id = id;
    }
    fn pre_handle_event(
        &mut self,
        mgr: &mut ::kas::event::EventMgr,
        event: ::kas::event::Event,
    ) -> ::kas::event::Response {
        self.handle_event(mgr, event)
    }
}
